#include "Components/MujocoGeomComponent.h"
#include "Components/MujocoBodyComponent.h"
#include "PhysicsEngine/BodySetup.h"
#if WITH_EDITORONLY_DATA
	#include "AssetRegistry/AssetRegistryModule.h"
#endif

UMujocoGeomComponent::UMujocoGeomComponent()
{
	PrimaryComponentTick.bCanEverTick = false;
}

#if WITH_EDITORONLY_DATA
void UMujocoGeomComponent::OnComponentCreated()
{
	Super::OnComponentCreated();
	if (GEngine && GetWorld())
	{
		if (UMujocoBodyComponent* ParentBody = Cast<UMujocoBodyComponent>(GetAttachParent()))
		{
			FString BodyName = ParentBody->GetName();
			FString UniqueName = GenerateUniqueName(GetWorld(), this, BodyName + TEXT("_Geom"));
			Rename(*UniqueName);
		}
		else
		{
			FString UniqueName = GenerateUniqueName(GetWorld(), this, GetName());
			Rename(*UniqueName);
		}
	}
}

void UMujocoGeomComponent::PostEditChangeProperty(FPropertyChangedEvent& PropertyChangedEvent)
{
	Super::PostEditChangeProperty(PropertyChangedEvent);
	if (GEngine && GetWorld())
	{
		FString UniqueName = GenerateUniqueName(GetWorld(), this, GetName());
		Rename(*UniqueName);
	}

	if (PropertyChangedEvent.Property && PropertyChangedEvent.Property->GetFName() == GET_MEMBER_NAME_CHECKED(UMujocoGeomComponent, MujocoStaticMesh))
	{
		if (MujocoStaticMesh && !MujocoStaticMesh->IsA<UMujocoStaticMesh>())
		{
			FString OriginalName = MujocoStaticMesh->GetName();
			FString StaticMeshPackagePath = FPackageName::GetLongPackagePath(MujocoStaticMesh->GetOutermost()->GetName());
			FString NewAssetName = FString::Printf(TEXT("%s_Mujoco"), *OriginalName);
			
			if (StaticMeshPackagePath.StartsWith("/Engine"))
			{
				StaticMeshPackagePath = StaticMeshPackagePath.Replace(TEXT("/Engine"), TEXT("/Game"));
			}

			const FString NewPackageName = FString::Printf(TEXT("%s/%s"), *StaticMeshPackagePath, *NewAssetName);
			UPackage* NewPackage = CreatePackage(*NewPackageName);
			UMujocoStaticMesh* NewMujocoMesh = NewObject<UMujocoStaticMesh>(NewPackage, UMujocoStaticMesh::StaticClass(), *NewAssetName, RF_Public | RF_Standalone | RF_Transactional);

			if (FMeshDescription* OriginalMeshDesc = MujocoStaticMesh->GetMeshDescription(0))
			{				
				NewMujocoMesh->SetNumSourceModels(1);
				FStaticMeshSourceModel& SourceModelLOD0 = NewMujocoMesh->GetSourceModel(0);
				SourceModelLOD0.BuildSettings.bRecomputeNormals = true;
				SourceModelLOD0.BuildSettings.bRecomputeTangents = true;
				SourceModelLOD0.BuildSettings.bBuildReversedIndexBuffer = true;
				SourceModelLOD0.BuildSettings.bGenerateLightmapUVs = false;
				
				NewMujocoMesh->SetStaticMaterials(MujocoStaticMesh->GetStaticMaterials());
				NewMujocoMesh->BuildFromMeshDescriptions({ OriginalMeshDesc });

				NewMujocoMesh->SetLightingGuid(FGuid::NewGuid());
				NewMujocoMesh->ImportVersion = EImportStaticMeshVersion::LastVersion;
				NewMujocoMesh->PostEditChange();
			}

			FAssetRegistryModule::AssetCreated(NewMujocoMesh);
			NewPackage->MarkPackageDirty();
			MujocoStaticMesh = NewMujocoMesh;
		}
	}
}

void UMujocoGeomComponent::PostInitProperties()
{
	Super::PostInitProperties();
}

void UMujocoGeomComponent::PreStaticMeshCompilation()
{
	Super::PreStaticMeshCompilation();
}

void UMujocoGeomComponent::PostStaticMeshCompilation()
{
	Super::PostStaticMeshCompilation();
}

void UMujocoGeomComponent::OnRegister()
{
	Super::OnRegister();
}

#endif
