#include "Misc/MujocoStaticMesh.h"

#if WITH_EDITORONLY_DATA
void UMujocoStaticMesh::PostEditChangeProperty(FPropertyChangedEvent& PropertyChangedEvent) 
{
	Super::PostEditChangeProperty(PropertyChangedEvent);	

	RefreshObjFileData();
}

void UMujocoStaticMesh::RefreshObjFileData() 
{
	FString GeneratedObj = TEXT("# OBJ file generated by LuckyMujoco\n");

	if (LodIndex >= GetNumLODs())
	{
		LodIndex = GetNumLODs() - 1;
	}

	const FStaticMeshLODResources& LODResources = GetLODForExport(LodIndex);

	TArray<FVector> Vertices;
	TArray<FVector> Normals;

	const uint32 TriangleCount = LODResources.GetNumTriangles();

	Vertices.Reserve(3 * TriangleCount);
	Normals.Reserve(3 * TriangleCount);
	FIndexArrayView IndexArray = LODResources.IndexBuffer.GetArrayView();
	const FQuat Rot(0.707107, 0, 0, -0.707107);
	for (uint32 TriIdx = 0; TriIdx < TriangleCount; ++TriIdx)
	{
		const uint32 Index0 = IndexArray[(TriIdx * 3) + 0];
		const uint32 Index1 = IndexArray[(TriIdx * 3) + 1];
		const uint32 Index2 = IndexArray[(TriIdx * 3) + 2];

		const FVector Vert0 = (FVector)LODResources.VertexBuffers.PositionVertexBuffer.VertexPosition(Index0) * ObjScale;
		const FVector Vert1 = (FVector)LODResources.VertexBuffers.PositionVertexBuffer.VertexPosition(Index1) * ObjScale;
		const FVector Vert2 = (FVector)LODResources.VertexBuffers.PositionVertexBuffer.VertexPosition(Index2) * ObjScale;

		Vertices.Add(Rot.RotateVector(Vert0));
		Vertices.Add(Rot.RotateVector(Vert1));
		Vertices.Add(Rot.RotateVector(Vert2));

		Normals.Add(Rot.RotateVector(FVector(LODResources.VertexBuffers.StaticMeshVertexBuffer.VertexTangentZ(Index0))));
		Normals.Add(Rot.RotateVector(FVector(LODResources.VertexBuffers.StaticMeshVertexBuffer.VertexTangentZ(Index1))));
		Normals.Add(Rot.RotateVector(FVector(LODResources.VertexBuffers.StaticMeshVertexBuffer.VertexTangentZ(Index2))));
	}

	for (int32 i = 0; i < Vertices.Num(); i++)
	{
		GeneratedObj += FString::Printf(TEXT("v %.6e %.6e %.6e\n"), Vertices[i].X, Vertices[i].Z, Vertices[i].Y);
	}

	for (int32 i = 0; i < Normals.Num(); i++)
	{
		GeneratedObj += FString::Printf(TEXT("vn %.6e %.6e %.6e\n"), Normals[i].X, Normals[i].Z, Normals[i].Y);
	}

	for (uint32 TriIdx = 0; TriIdx < TriangleCount; ++TriIdx)
	{
		const uint32 Index0 = TriIdx * 3 + 1;
		const uint32 Index1 = TriIdx * 3 + 2;
		const uint32 Index2 = TriIdx * 3 + 3;

		GeneratedObj += FString::Printf(TEXT("f %d//%d %d//%d %d//%d\n"), Index0, Index0, Index1, Index1, Index2, Index2);
	}

	ObjFileData = MoveTemp(GeneratedObj);
}
#endif
