#pragma once

#include "CoreMinimal.h"
#include "MujocoEnums.generated.h"

UENUM(BlueprintType)
enum class EMujocoJointType : uint8
{	
	Free UMETA(DisplayName = "Free"),
	Ball UMETA(DisplayName = "Ball"),
	Slide UMETA(DisplayName = "Slide"),
	Hinge UMETA(DisplayName = "Hinge")
};

UENUM(BlueprintType)
enum class EMujocoShapeType : uint8
{
	Sphere UMETA(DisplayName = "Sphere"),
	Capsule UMETA(DisplayName = "Capsule"),
	Ellipsoid UMETA(DisplayName = "Ellipsoid"),
	Cylinder UMETA(DisplayName = "Cylinder"),
	Box UMETA(DisplayName = "Box")
};

UENUM(BlueprintType)
enum class ELimitType : uint8
{
	False_ UMETA(DisplayName = "False"),
	True_ UMETA(DisplayName = "True"),
	Auto_ UMETA(DisplayName = "Auto")
};

UENUM(BlueprintType)
enum class EDynType : uint8
{
	None UMETA(DisplayName = "None"),
	Integrator UMETA(DisplayName = "Integrator"),
	Filter UMETA(DisplayName = "Filter"),
	FilterExact UMETA(DisplayName = "FilterExact"),
	Muscle UMETA(DisplayName = "Muscle"),
	User UMETA(DisplayName = "User")
};

UENUM(BlueprintType)
enum class EGainType : uint8
{
	Fixed UMETA(DisplayName = "Fixed"),
	Affine UMETA(DisplayName = "Affine"),
	Muscle UMETA(DisplayName = "Muscle"),
	User UMETA(DisplayName = "User")
};

UENUM(BlueprintType)
enum class EBiasType : uint8
{
	None UMETA(DisplayName = "None"),
	Affine UMETA(DisplayName = "Affine"),
	Muscle UMETA(DisplayName = "Muscle"),
	User UMETA(DisplayName = "User")
};

UENUM(BlueprintType)
enum class EMujocoTendonType : uint8
{
	Spatial UMETA(DisplayName = "Spatial"),
	Fixed UMETA(DisplayName = "Fixed")
};

UENUM(BlueprintType)
enum class EMujocoEqualityType : uint8
{
	Connect UMETA(DisplayName = "Connect"),
	Weld UMETA(DisplayName = "Weld"),
	Joint UMETA(DisplayName = "Joint"),
	Tendon UMETA(DisplayName = "Tendon"),
	Flex UMETA(DisplayName = "Flex")
};

UENUM(BlueprintType)
enum class EMujocoContactDim : uint8
{
	Invalid = 0,
	Frictionless = 1,
	Regular = 3,
	Soft = 4,
	Rolling = 6
};

USTRUCT(BlueprintType)
struct FMujocoFriction
{
	GENERATED_BODY()

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Friction")
	float Sliding = 1.0f;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Friction")
	float Torsional = 0.005f;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Friction")
	float Rolling = 0.0001f;

	FString ToString() const { return FString::Printf(TEXT("%f %f %f"), Sliding, Torsional, Rolling); }
};

USTRUCT(BlueprintType)
struct FMujocoSolImp
{
	GENERATED_BODY()

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | SolImp")
	float DMin = 0.9f;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | SolImp")
	float DMax = 0.95f;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | SolImp")
	float Width = 0.001f;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | SolImp")
	float Midpoint = 0.5f;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | SolImp")
	float Power = 2.0f;

	FString ToString() const { return FString::Printf(TEXT("%f %f %f %f %f"), DMin, DMax, Width, Midpoint, Power); }
};

USTRUCT(BlueprintType)
struct FMujocoSolRef
{
	GENERATED_BODY()

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | SolRef")
	float TimeConst = 0.02f;
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | SolRef")
	float DampRatio = 1.0f;

	FString ToString() const { return FString::Printf(TEXT("%f %f"), TimeConst, DampRatio); }
};

USTRUCT(BlueprintType)
struct FMujocoRelPose
{
	GENERATED_BODY()

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | RelPose")
	FVector Pos = FVector::ZeroVector;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | RelPose")
	FVector Rot = FVector::ZeroVector;

	FString ToString() const { return FString::Printf(TEXT("%.6e %.6e %.6e %.6e %.6e %.6e"), Pos.X, Pos.Y, Pos.Z, Rot.X, Rot.Y, Rot.Z); }
};

USTRUCT(BlueprintType)
struct FMujocoPolyCoef
{
	GENERATED_BODY()

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | PolyCoef")
	float A0 = 0.0f;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | PolyCoef")
	float A1 = 0.0f;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | PolyCoef")
	float A2 = 0.0f;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | PolyCoef")
	float A3 = 0.0f;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | PolyCoef")
	float A4 = 0.0f;

	FString ToString() const { return FString::Printf(TEXT("%f %f %f %f %f"), A0, A1, A2, A3, A4); }
};

USTRUCT(BlueprintType)
struct FMujocoGear
{
	GENERATED_BODY()

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Gear")
	FVector G0 = FVector::ForwardVector;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Gear")
	FVector G1 = FVector::ZeroVector;

	// to string override
	FString ToString() const { return FString::Printf(TEXT("%.6e %.6e %.6e %.6e %.6e %.6e"), G0.X, G0.Y, G0.Z, G1.X, G1.Y, G1.Z); }
};

UENUM(BlueprintType)
enum class EMujocoActuatorType : uint8
{
	General UMETA(DisplayName = "general"),
	Motor UMETA(DisplayName = "motor"),
	Position UMETA(DisplayName = "position"),
	Velocity UMETA(DisplayName = "velocity"),
	IntVelocity UMETA(DisplayName = "intvelocity"),
	Damper UMETA(DisplayName = "damper"),
	Cylinder UMETA(DisplayName = "cylinder"),
	Muscle UMETA(DisplayName = "muscle"),
	Adhesion UMETA(DisplayName = "adhesion")
};

USTRUCT(BlueprintType)
struct FMujocoTendonSpatialSite
{
	GENERATED_BODY()

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon | Spatial", meta = (Attribute = "site"))
	FName Site = NAME_None;
};

USTRUCT(BlueprintType)
struct FMujocoTendonSpatialGeom
{
	GENERATED_BODY()

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon | Spatial", meta = (Attribute = "geom"))
	FName Geom = NAME_None;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon | Spatial", meta = (Attribute = "sidesite"))
	FName SideSite = NAME_None;
};

USTRUCT(BlueprintType)
struct FMujocoTendonPulley
{
	GENERATED_BODY()

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon | Spatial", meta = (Attribute = "divisor"))
	TOptional<double> Divisor;
};

USTRUCT(BlueprintType)
struct FMujocoTendonFixedJoint
{
	GENERATED_BODY()

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon | Fixed", meta = (Attribute = "joint"))
	FName Joint = NAME_None;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon | Fixed", meta = (Attribute = "site"))
	TOptional<double> Coef;
};