#pragma once

#include "CoreMinimal.h"
#include "Enums/MujocoEnums.h"
#include "MujocoEquality.generated.h"

USTRUCT(BlueprintType)
struct FMujocoEquality
{
	GENERATED_BODY()	

	/** Type of the equality constraint. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality")
	EMujocoEqualityType Type = EMujocoEqualityType::Joint;	

	/** If true, the constraint is active and the constraint solver will try to enforce it. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality", meta = (Attribute = "active"))
	TOptional<bool> Active;

	/** Name of the first body participating in the constraint. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality", meta = (EditConditionHides, EditCondition = "Type == EMujocoEqualityType::Connect || Type == EMujocoEqualityType::Weld", Attribute = "body1"))
	FName Body1 = NAME_None;

	/** Name of the second body participating in the constraint. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality", meta = (EditConditionHides, EditCondition = "Type == EMujocoEqualityType::Connect || Type == EMujocoEqualityType::Weld", Attribute = "body2"))
	FName Body2 = NAME_None;

	/** Name of a site belonging to the first body participating in the constraint. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality", meta = (EditConditionHides, EditCondition = "Type == EMujocoEqualityType::Connect || Type == EMujocoEqualityType::Weld", Attribute = "site1"))
	FName Site1 = NAME_None;

	/** Name of a site belonging to the second body participating in the constraint. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality", meta = (EditConditionHides, EditCondition = "Type == EMujocoEqualityType::Connect || Type == EMujocoEqualityType::Weld", Attribute = "site2"))
	FName Site2 = NAME_None;

	/** Name of the first joint. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality", meta = (EditConditionHides, EditCondition = "Type == EMujocoEqualityType::Joint", Attribute = "joint1"))
	FName Joint1 = NAME_None;

	/** Name of the second joint. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality", meta = (EditConditionHides, EditCondition = "Type == EMujocoEqualityType::Joint", Attribute = "joint2"))
	FName Joint2 = NAME_None;

	/** Name of the first tendon. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality", meta = (EditConditionHides, EditCondition = "Type == EMujocoEqualityType::Tendon", Attribute = "tendon1"))
	FName Tendon1 = NAME_None;

	/** Name of the second tendon. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality", meta = (EditConditionHides, EditCondition = "Type == EMujocoEqualityType::Tendon", Attribute = "tendon2"))
	FName Tendon2 = NAME_None;

	/** Name of the flex whose edges are being constrained. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality", meta = (EditConditionHides, EditCondition = "Type == EMujocoEqualityType::Flex", Attribute = "flex"))
	FName Flex = NAME_None;

	/** Constraint solver parameters for equality constraint simulation. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality", meta = (Attribute = "solref"))
	TOptional<FMujocoSolRef> SolRef;

	/** Constraint solver parameters for equality constraint simulation. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality", meta = (Attribute = "solimp"))
	TOptional<FMujocoSolImp> SolImp;

	/** Coordinates of the 3D anchor point where the two bodies are connected. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality", meta = (Attribute = "anchor"))
    TOptional<FVector> Anchor;

	/** Specifies the relative pose of body2 relative to body1. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality", meta = (EditConditionHides, EditCondition = "Type == EMujocoEqualityType::Weld", Attribute = "relpose"))
    TOptional<FMujocoRelPose> RelPose;

	/** A constant that scales the angular residual (angular constraint violation). */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality", meta = (EditConditionHides, EditCondition = "Type == EMujocoEqualityType::Weld", Attribute = "torquescale"))
    TOptional<double> TorqueScale;

	/** Coefficients of the quartic polynomial. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Equality", meta = (EditConditionHides, EditCondition = "Type == EMujocoEqualityType::Joint || Type == EMujocoEqualityType::Tendon", Attribute = "polycoef"))
    TOptional<FMujocoPolyCoef> PolyCoef;	
};