#pragma once

#include "CoreMinimal.h"
#include "Enums/MujocoEnums.h"
#include "MujocoTendon.generated.h"

USTRUCT(BlueprintType)
struct FMujocoTendon
{
	GENERATED_BODY()	

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon")
	EMujocoTendonType Type = EMujocoTendonType::Spatial;	

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon", meta = (Attribute = "name"))
	FName Name = NAME_None;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon", meta = (Attribute = "group", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Spatial"))
	TOptional<int32> Group;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon", meta = (Attribute = "limited", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Spatial"))
	TOptional<ELimitType> Limited;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon", meta = (Attribute = "range", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Spatial"))
    TOptional<FVector2D> Range;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon", meta = (Attribute = "solreflimit", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Spatial"))
    TOptional<double> SolRefLimit;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon", meta = (Attribute = "solimplimit", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Spatial"))
    TOptional<double> SolImpLimit;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon", meta = (Attribute = "solreffriction", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Spatial"))
    TOptional<double> SolRefFriction;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon", meta = (Attribute = "solimpfriction", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Spatial"))
    TOptional<double> SolImpFriction;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon", meta = (Attribute = "margin", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Spatial"))
    TOptional<double> Margin;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon", meta = (Attribute = "frictionloss", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Spatial"))
    TOptional<double> FrictionLoss;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon", meta = (Attribute = "width", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Spatial"))
    TOptional<double> Width;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon", meta = (Attribute = "springlength", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Spatial"))
    TOptional<FVector2D> SpringLength;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon", meta = (Attribute = "stiffness", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Spatial"))
    TOptional<double> Stiffness;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon", meta = (Attribute = "damping", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Spatial"))
    TOptional<double> Damping;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon | Spatial", meta = (Attribute = "site", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Spatial"))
    TArray<FMujocoTendonSpatialSite> SpatialSite;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon | Spatial", meta = (Attribute = "geom", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Spatial"))
	TArray<FMujocoTendonSpatialGeom> SpatialGeom;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon | Spatial", meta = (Attribute = "pulley", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Spatial"))
	TArray<FMujocoTendonPulley> Pulley;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Tendon | Fixed", meta = (Attribute = "fixed", EditConditionHides, EditCondition = "Type == EMujocoTendonType::Fixed"))
	TArray<FMujocoTendonFixedJoint> FixedJoint;
};